--[[

Работа с сущностями

--]]

function Z:DetailEntity(entity)
  local obj = {}

  function obj.id()
    return entity
  end

  -- Текущий уровень HP сущности
  function obj.hp(value)
    local component = EntityGetFirstComponent(entity, "DamageModelComponent")
    if value then
      ComponentSetValue(component, "hp", value/25)
    else
      return ComponentGetValueFloat(component, "hp") * 25
    end
  end

  -- Максимальный уровень HP сущности
  function obj.max_hp(value)
    local component = EntityGetFirstComponent(entity, "DamageModelComponent")
    if value then
      ComponentSetValue(component, "max_hp", value/25)
    else
      return ComponentGetValue(component, "max_hp") * 25
    end
  end

  -- Компоненты
  -- если tag == nil, то редактирует изначальный компонент
  function obj.component(tag, component, table, value)
    local _component = ""
    -- Сеттер
    if value then
      if tag then 
        if EntityGetFirstComponent(entity, component, tag) == nil then
          _component = EntityAddComponent(entity, component,  {_tags = tag})
        else 
          _component = EntityGetFirstComponent(entity, component, tag)
        end
      else
        if EntityGetFirstComponent(entity, component) == nil then
          _component = EntityAddComponent(entity, component)
        else 
          _component = EntityGetFirstComponent(entity, component)
        end
      end
      ComponentSetValue(_component, table, value)
      return _component
      
    -- Геттер
    else
      if tag then
        _component = EntityGetFirstComponent(entity, component, tag)
      else 
        _component = EntityGetFirstComponent(entity, component)
      end
      return ComponentGetValue(_component, table)
    end
  end

  -- Теги
  function obj.tag(tag)
    if tag ~= nil then
      EntityAddTag(entity, tag)
    else
      return EntityGetTags(entity)
    end
  end

  function obj.is_tag(tag)
    return EntityHasTag(entity, tag)
  end

  setmetatable(obj, self)
  self.__index = self 
  return obj

end

function Z:Entity(tag, x, y, radius)
  local obj = {}
  local entities = EntityGetWithTag(tag)
  if x and y and radius then
    entities = EntityGetInRadiusWithTag(x, y, radius, tag)
  end

  for i, entity in ipairs(entities) do
    obj[i] = Z:DetailEntity(entity)
  end

  setmetatable(obj, self)
  self.__index = self 
  return obj
end